<?php
namespace Ep\Woo\Plugin\Payment;

use Ak\Wp\Plugin\PluginInterface;

/**
 * Class TwoCheckout
 *
 * @package Ep\Woo\Plugin\Payment
 * @author Elektronika Perhoč <web@ep.hr>
 * @link http://gitlab.ep.local/ep-framework/woocommerce
 */
class TwoCheckout extends \WC_Payment_Gateway implements PluginInterface
{
    const CHECKOUT_URL = 'https://www.2checkout.com/checkout/purchase';
    const SANDBOX_CHECKOUT_URL = 'https://sandbox.2checkout.com/checkout/purchase';

    /** @var bool */
    protected $sandbox = false;
    /** @var int */
    protected $blockId;
    /** @var string */
    protected $lang;
    /** @var int */
    protected $successPageId;
    /** @var string */
    protected $secretWord;
    /** @var string */
    protected $sid;

    /**
     * TwoCheckout constructor.
     */
    public function __construct()
    {
        if (!function_exists('get_option')) {
            return;
        }

        $this->id = 'two_checkout';
        $this->method_title = '2Checkout';
        $this->enabled = $this->get_option('enabled') === 'yes';

        $this->init_form_fields();
        $this->init_settings();

        $this->icon = apply_filters('ep_asset_url', 'img/twocheckout.png');
        $this->title = $this->get_option('title');
        $this->lang = $this->get_option('lang');
        $this->blockId = intval($this->get_option('block'));
        $this->description = $this->get_option('description');
        $this->successPageId = intval($this->get_option('success'));
        $this->sandbox = $this->get_option('is_test') === 'yes';
        $this->sid = $this->sandbox ? $this->get_option('sandbox_sid') : $this->get_option('sid');
        $this->secretWord = $this->sandbox ? $this->get_option('sandbox_secret_word') : $this->get_option('secret_word');

        add_action("woocommerce_receipt_{$this->id}", [$this, 'receipt_page']);
        add_action("woocommerce_update_options_payment_gateways_{$this->id}", [$this, 'process_admin_options']);
    }

    /**
     * Dodaje akcije i filtere u WordPress.
     *
     * @return void
     */
    public function attach(): void
    {
        add_filter('woocommerce_payment_gateways', function($gateways) {
            $gateways[] = self::class;
            return $gateways;
        });

        add_action('init', [$this, 'validate_payment']);
    }

    public function validate_payment()
    {
        $this->__construct();

        if (isset($_GET['invoice_id'])) {
            $key = strtoupper(md5($this->secretWord . $this->sid . $_GET['order_number'] . $_GET['total']));

            if ($key === $_GET['key'] && $_GET['credit_card_processed'] === 'Y') {
                $orderId = intval($_GET['merchant_order_id']);
                $order = wc_get_order($orderId);
                $order->update_status('processing', sprintf('Invoice ID: %d', $_GET['invoice_id']));
            }
        }
    }

    public function init_form_fields()
    {
        if (!is_admin()) {
            return;
        }

        /** @var \WP_Post[] $blocks */
        $blocks = get_posts([
            'posts_per_page' => -1,
            'post_status' => 'publish',
            'post_type' => 'blocks',
        ]);

        /** @var \WP_Post[] $pages */
        $pages = get_posts([
            'posts_per_page' => -1,
            'post_status' => 'publish',
            'post_type' => 'page',
        ]);

        $blockOptions = ['' => '-'];
        foreach ($blocks as $block) {
            $blockOptions[$block->ID] = $block->post_title;
        }

        $pageOptions = ['' => '-'];
        foreach ($pages as $page) {
            $pageOptions[$page->ID] = $page->post_title;
        }

        $this->form_fields = [
            'enabled' => [
                'title' => 'Omogući/Onemogući',
                'type' => 'checkbox',
                'label' => 'Omogući 2Checkout naplatu',
                'default' => 'no',
            ],
            'title' => [
                'title' => 'Naziv',
                'type' => 'text',
                'description' => 'Ovo će korisnici vidjeti na stranici naplate',
                'default' => 'Kreditna kartica',
                'desc_tip' => true,
            ],
            'description' => [
                'title' => 'Opis',
                'type' => 'textarea',
            ],
            'lang' => [
                'title' => 'Jezik',
                'type' => 'select',
                'options' => [
                    'en' => 'English',
                    'zh' => 'Chinese',
                    'da' => 'Danish',
                    'nl' => 'Dutch',
                    'fr' => 'French',
                    'gr' => 'German',
                    'el' => 'Greek',
                    'it' => 'Italian',
                    'jp' => 'Japanese',
                    'no' => 'Norwegian',
                    'pt' => 'Portuguese',
                    'sl' => 'Slovenian',
                    'es_ib' => 'Spanish (European)',
                    'es_la' => 'Spanish (Latin American)',
                    'sv' => 'Swedish',
                ],
                'default' => 'en',
            ],
            'block' => [
                'title' => 'Blok na stranici s koje preusmjeravamo',
                'type' => 'select',
                'options' => $blockOptions,
            ],
            'success' => [
                'title' => 'Stranica nakon uspješne kupovine',
                'type' => 'select',
                'options' => $pageOptions,
            ],
            'sid' => [
                'title' => 'Broj računa',
                'type' => 'text',
            ],
            'secret_word' => [
                'title' => 'Tajna riječ',
                'type' => 'text',
            ],
            'is_test' => [
                'title' => 'Sandbox?',
                'type' => 'checkbox',
                'label' => 'Da li testiramo naplatu?',
                'default' => 'no',
            ],
            'sandbox_sid' => [
                'title' => 'Broj računa u sandboxu',
                'type' => 'text',
            ],
            'sandbox_secret_word' => [
                'title' => 'Tajna riječ u sandboxu',
                'type' => 'text',
            ],
        ];
    }

    public function receipt_page($orderId)
    {
        if ($this->blockId > 0) {
            $block = get_post($this->blockId);
            echo apply_filters('the_content', $block->post_content);
        }

        echo $this->generate_form($orderId);
    }

    public function process_payment($order_id)
    {
        $order = wc_get_order($order_id);

        return [
            'result' => 'success',
            'redirect' => $order->get_checkout_payment_url(true),
        ];
    }

    protected function get_checkout_url()
    {
        return $this->sandbox
            ? self::SANDBOX_CHECKOUT_URL
            : self::CHECKOUT_URL;
    }

    protected function generate_form($orderId)
    {
        $order = wc_get_order($orderId);

        $params = [
            'sid' => $this->sid,
            'mode' => '2CO',
            'currency_code' => get_woocommerce_currency(),
            'lang' => $this->lang,
            'merchant_order_id' => $orderId,
            'purchase_step' => 'payment-method',
            'x_receipt_link_url' => get_permalink($this->successPageId),
        ];

        $itemCount = 0;
        /** @var \WC_Order_Item[] $items */
        $items = $order->get_items(['line_item', 'shipping']);

        foreach ($items as $item) {
            $params['li_' . $itemCount . '_type'] = $item->get_type() === 'line_item'
                ? 'product' : $item->get_type();
            $params['li_' . $itemCount . '_name'] = $item->get_name();
            $params['li_' . $itemCount . '_quantity'] = $item->get_quantity();

            if ($item instanceof \WC_Order_Item_Product) {
                $params['li_' . $itemCount . '_price'] = round(floatval($item->get_total() + $item->get_total_tax()) / $item->get_quantity(), 2);
                $params['li_' . $itemCount . '_tangible'] = 'Y';
            } else if ($item instanceof \WC_Order_Item_Tax) {
                $params['li_' . $itemCount . '_price'] = round($item->get_tax_total(), 2);
                $params['li_' . $itemCount . '_tangible'] = 'N';
            } else if ($item instanceof \WC_Order_Item_Shipping) {
                $params['li_' . $itemCount . '_price'] = round($item->get_total() + $item->get_total_tax(), 2);
                $params['li_' . $itemCount . '_tangible'] = 'Y';
            }

            $itemCount++;
        }

        /** @noinspection PhpUnhandledExceptionInspection */
        $customer = new \WC_Customer(get_current_user_id());

        $params['card_holder_name'] = $customer->get_billing_first_name() . ' ' . $customer->get_billing_last_name();
        $params['street_address'] = $customer->get_billing_address_1();
        $params['street_address2'] = $customer->get_billing_address_2();
        $params['city'] = $customer->get_billing_city();
        $params['zip'] = $customer->get_billing_postcode();
        $params['country'] = $customer->get_billing_country();
        $params['email'] = $customer->get_billing_email();
        $params['phone'] = $customer->get_billing_phone();

        $params['ship_name'] = $customer->get_shipping_first_name() . ' ' . $customer->get_shipping_last_name();
        $params['ship_street_address'] = $customer->get_shipping_address_1();
        $params['ship_street_address2'] = $customer->get_shipping_address_2();
        $params['ship_city'] = $customer->get_shipping_city();
        $params['ship_zip'] = $customer->get_shipping_postcode();
        $params['ship_country'] = $customer->get_shipping_country();

        /** @noinspection HtmlUnknownTarget */
        $form = sprintf('<form id="twocheckoutform" action="%s" method="post">', $this->get_checkout_url());

        foreach ($params as $name => $value) {
            $form .= sprintf('<input type="hidden" name="%s" value="%s">', $name, $value);
        }

        $form .= '<button type="submit">' . __('Continue', 'woocommerce') . '</button></form>';
        $form .= '<script>document.getElementById("twocheckoutform").submit();</script>';

        return $form;
    }
}